// FRAP analysis macro for educational purposes - alternative solution
// 
// you find a more advanced version online:
// http://imagej.net/Analyze_FRAP_movies_with_a_Jython_script
//
// Usage:
// * Run FIJI (download it from http://fiji.sc)
// * Open your image
// * Open this script
// * Click the run button
//
// Author: Robert Haase, Scientific Computing Facility, MPI CBG Dresden
//         rhaase@mpi-cbg.de
// 
////////////////////////////////////////////////////////////////

// empty the results table to prevent result mixing
run("Clear Results");

// register the stack to correct for motion of the sample
// run("StackReg", "transformation=Translation");

// ask the user to select a region of interest
setTool("oval");
waitForUser("Please draw a circle in the frapped region.");

// read out size of the image stack (in time)
getDimensions(width, height, channels, slices, frames);

// signal readout
measurements = newArray(frames);
for (i = 0; i < frames; i++) {
	Stack.setFrame(i + 1); // add one, because frames start at 1
	
	getStatistics(area, mean, min, max, stdDev);

	// print intermediat results
    //print("Mean signal in frame " + i + " is " + mean);

	measurements[i] = mean;
}

// determine initial and final signal
initialSignal = measurements[0];
finalSignal   = measurements[frames - 1];

// determine minimum signal
minimumSignal = initialSignal;
minimumSignalTimePoint = 1;

for (i = 0; i < frames; i++) {
	if (measurements[i] < minimumSignal) {
		minimumSignal = measurements[i];
		minimumSignalTimePoint = i + 1; // add 1, because frames start at 1
	}
}

// print intermediat results
//print("initialSignal: " + initialSignal);
//print("minimumSignal: " + minimumSignal);
//print("finalSignal:   " + finalSignal);

immobileFraction = (initialSignal - finalSignal)
                   / (initialSignal - minimumSignal);

mobileFraction = (finalSignal - minimumSignal)
                 / (initialSignal - minimumSignal);

// print intermediat results
//print("immobileFraction:   " + (immobileFraction * 100) + "%");
//print("mobileFraction:   " + (mobileFraction * 100) + "%");

// get the characteristic diffusion time
halfRecoveriedSignal = minimumSignal + 
                       (finalSignal - minimumSignal) / 2.0;

// print intermediat results
//print("halfRecoveriedSignal " + halfRecoveriedSignal);

// look for the time point when half of the signal has recovered
for (i = minimumSignalTimePoint; i < frames; i++) {
	if (measurements[i] > halfRecoveriedSignal) {
		halfRecoveriedSignalTimePoint = i;
		break;
	}
}

// print intermediat results
//print("halfRecoveriedSignalTimePoint " + halfRecoveriedSignalTimePoint);

characteristicDiffusionTime = halfRecoveriedSignalTimePoint - minimumSignalTimePoint;

// finally output the results of the analysis
print("\\Clear");
print("Analysis of " + getTitle());
print("----------------------------"); 
print("IF: " + (immobileFraction * 100) + "%");
print("MF: " + (mobileFraction * 100) + "%");
print("CDT: " + characteristicDiffusionTime + " frames");







