// Visualise large and small objects differently
// Jython_script
//
// Usage:
// * Run FIJI (download it from http://fiji.sc)
// * Change the location of the image file below this paragraph
// * Open this script
// * Click the run button
//
// Author: Robert Haase, Scientific Computing Facility, MPI CBG Dresden
//         rhaase@mpi-cbg.de
// 
////////////////////////////////////////////////////////////////
open("/Users/rhaase/temp/blobs.gif");

run("Duplicate...", " ");

// apply threshold
setAutoThreshold("Default");
setOption("BlackBackground", true);
run("Convert to Mask");

// create ROI(s)
run("Create Selection");
run("ROI Manager...");
roiManager("Split");
close();

// determine average ROI area
sum = 0; 
for(i = 0; i < roiManager("count"); i++) {
    roiManager("Select", i);
    getStatistics(area, mean, min, max, standard_deviation);
    sum += area;
}
averageArea = sum / roiManager("count");

// delete ROIs smaller than average
for (i = 0; i < roiManager("count"); i++) {
    roiManager("Select", i);
    getStatistics(area, mean, min, max, standard_deviation);

    if (area < averageArea) {
        Overlay.addSelection("red");    
    } else { 
        Overlay.addSelection("green");	    
    }
}

run("Set Measurements...", 
    "area perimeter redirect=None decimal=4");
roiManager("Deselect");
roiManager("Measure");

for (row = 0; row < nResults(); row++) {
   
    // read from table
    area = getResult("Area", row);
    perimeter = getResult("Perim.", row);


    // calculate another shape descriptor
    circularity = 4 * PI * area / pow(perimeter, 2);

    // write into table
    setResult("Circularity", row, circularity );
}

